@@ -3,30 +3,73 @@ package App::PAIA::Command;
 use strict;
 use v5.10;
 use App::Cmd::Setup -command;
-our $VERSION = '0.20'; #VERSION
+our $VERSION = '0.21'; #VERSION
 
 use App::PAIA::Agent;
 use App::PAIA::JSON;
+use App::PAIA::File;
 use URI::Escape;
 use URI;
 
-sub config {
-    my ($self) = @_;
-    $self->{config} //= App::PAIA::JSON::File->new(
-        owner => $self,
-        type  => 'config',
-        file  => $self->app->global_options->config,
-    );
+# Implements lazy accessors just like Mo, Moo, Moose...
+sub has {
+    my ($name, %options) = @_;
+    my $default = $options{default};
+    no strict 'refs'; ## no critic 
+    *{__PACKAGE__."::$name"} = sub {
+        @_ > 1 
+            ? $_[0]->{$name} = $_[1]
+            : (!exists $_[0]->{$name} && $default)
+                ? $_[0]->{$name} = $default->($_[0])
+                : $_[0]->{$name}
+    }
 }
 
-sub session {
-    my ($self) = @_;
-    $self->{session} //= App::PAIA::JSON::File->new(
-        owner => $self,
-        type  => 'session',
-        file  => $self->app->global_options->session,
-    );
-}
+has config => ( 
+    default => sub {
+        App::PAIA::File->new(
+            logger => $_[0]->logger,
+            type   => 'config',
+            file   => $_[0]->app->global_options->config,
+        ) 
+    }
+);
+
+has session => ( 
+    default => sub { 
+        App::PAIA::File->new(
+            logger => $_[0]->logger,
+            type   => 'session',
+            file   => $_[0]->app->global_options->session,
+        ) 
+    }
+);
+
+has agent => (
+    default => sub {
+        App::PAIA::Agent->new(
+            insecure => $_[0]->option('insecure'),
+            logger   => $_[0]->logger,
+            dumper   => $_[0]->dumper,
+        );
+    }
+);
+
+has logger => (
+    default => sub {
+        $_[0]->app->global_options->verbose
+            ? sub { say "# $_" for split "\n", $_[0]; }
+            : sub { };
+    }
+);
+
+has dumper => (
+    default => sub {
+        $_[0]->app->global_options->full
+            ? sub { say "> $_" for split "\n", $_[0]; }
+            : sub { };
+    }
+);
 
 sub option { 
     my ($self, $name) = @_;
@@ -41,9 +84,6 @@ sub explicit_option {
         // $self->config->get($name);   # config file
 }
 
-# get base URL
-sub base { $_[0]->option('base') }
-
 # get auth URL
 sub auth { 
     my ($self) = @_;
@@ -56,14 +96,25 @@ sub core {
     $_[0]->option('core') // ( $self->base ? $self->base . '/core' : undef );
 }
 
+#has_option 'base';
+#has_option 'patron';
+
+# get base URL
+sub base { $_[0]->option('base') }
+
 # get patron identifier
 sub patron { $_[0]->option('patron') }
 
 # get current scopes
 sub scope { $_[0]->option('scope') }
 
-# get verbose mode
-sub verbose { $_[0]->option('verbose') }
+sub username {
+    $_[0]->explicit_option('username') // $_[0]->usage_error("missing username");
+}
+
+sub password {
+    $_[0]->explicit_option('password') // $_[0]->usage_error("missing password");
+}
 
 sub token {
     my ($self) = @_;
@@ -97,21 +148,6 @@ sub has_scope {
     return index($has_scope, $scope) != -1;
 }
 
-# emit a message only in verbose mode
-sub log {
-    my ($self, $msg, $verbose) = @_;
-    if ($verbose // $self->verbose) {
-        say "# $_" for split "\n", $msg;
-    }
-}
-
-sub agent {
-    my ($self) = @_;
-    $self->{agent} //= App::PAIA::Agent->new(
-        map { $_ => $self->option($_) } qw(insecure verbose quiet)
-    );
-}
-
 sub request {
     my ($self, $method, $url, $param) = @_;
 
@@ -123,13 +159,20 @@ sub request {
 
     my ($response, $json) = $self->agent->request( $method, $url, $param, %headers );
 
+    # handle request errors
+    if (ref $json and defined $json->{error}) {
+        my $msg = $json->{error};
+        if (defined $json->{error_description}) {
+            $msg .= ': '.$json->{error_description};
+        }
+        die "$msg\n";
+    }
+
     if ($response->{status} ne '200') {
         my $msg = $response->{content} // 'HTTP request failed: '.$response->{status};
         die "$msg\n";
     }
 
-    # TODO: more error handling
-
     if (my $scopes = $response->{headers}->{'x-oauth-scopes'}) {
         $self->session->set( scope => $scopes );
     }
@@ -144,8 +187,8 @@ sub login {
 
     # take credentials from command line or config file only
     my %params = (
-        username => ($self->explicit_option('username') // $self->usage_error("missing username")),
-        password => ($self->explicit_option('password') // $self->usage_error("missing password")),
+        username   => $self->username,
+        password   => $self->password,
         grant_type => 'password',
     );
 
@@ -171,10 +214,10 @@ sub login {
 
 our %required_scopes = (
     patron  => 'read_patron',
-    items   => 'read_item',
-    request => 'write_item',
-    renew   => 'write_item',
-    cancel  => 'write_item',
+    items   => 'read_items',
+    request => 'write_items',
+    renew   => 'write_items',
+    cancel  => 'write_items',
     fees    => 'read_fees',
     change  => 'change_password',
 );
@@ -187,7 +230,7 @@ sub auto_login_for {
     if ($self->not_authentificated( $scope )) {
         # add to existing scopes (TODO: only if wanted)
         my $new_scope = join ' ', split(' ',$self->scope // ''), $scope;
-        $self->log("auto-login with scope '$new_scope'");
+        $self->logger->("auto-login with scope '$new_scope'");
         $self->login( $new_scope );
         if ( $self->scope and !$self->has_scope($scope) ) {
             die "current scope does not include $scope!\n";
@@ -242,7 +285,9 @@ sub execute {
     }
 
     my $response = $self->_execute(@_);
-    print encode_json($response) if defined $response;
+    if (defined $response and !$self->app->global_options->quiet) {
+        print encode_json($response);
+    }
 }
 
 1;
@@ -258,7 +303,7 @@ App::PAIA::Command - common base class of PAIA client commands
 
 =head1 VERSION
 
-version 0.20
+version 0.21
 
 =head1 AUTHOR
 
